@@ -1,5 +1,16 @@
+#' @importFrom utils .DollarNames
+#' @export
+.DollarNames.rxEt <- function(x, pattern) {
+  grep(pattern, .Call(`_rxode2_etDollarNames`, x), value = TRUE)
+}
+
+.isRxEt <- function(obj) {
+  .Call(`_rxode2_rxIsEt2`, obj)
+}
+
+
 .etAddCls <- function(x) {
-  if (inherits(x, "rxEt")) {
+  if (.isRxEt(x)) {
     .x <- x
     .cls <- class(x)
     class(.x) <- "data.frame"
@@ -128,11 +139,14 @@
 #'     with arbitrary objects.
 #'
 #' @inheritParams base::eval
-#' @inheritParams rxSolve
+#' @inheritParams base::seq
 #' @return A new event table
 #'
 #' @template etExamples
-#'
+#' @importFrom Rcpp evalCpp
+#' @importFrom stats simulate end setNames start
+#' @importFrom utils assignInMyNamespace
+#' @importFrom methods is
 #' @export
 et <- function(x, ..., envir = parent.frame()) {
   UseMethod("et")
@@ -151,11 +165,125 @@ et <- function(x, ..., envir = parent.frame()) {
 .pipelineDfObs <- NULL
 .pipelineDfSub <- NULL
 .pipelineNSub <- NULL
+
+.isNa1 <- function(x) {
+  if (inherits(x, "logical") ||
+        inherits(x, "numeric") ||
+        inherits(x, "integer")) {
+    if (length(x) == 1) {
+      return(is.na(x))
+    }
+  }
+  FALSE
+}
+
 .pipelineNStud <- NULL
 
+#' Assign in the rxode2 pipeline
+#'
+#'
+#' @param obj  Object to assign.  If NA return the value.
+#' @return The pipeline object (invisibly)
+#' @author Matthew L. Fidler
+#' @export
+#' @keywords internal
+.pipeRx <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineRx))
+  assignInMyNamespace(".pipelineRx", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeInits <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineInits))
+  assignInMyNamespace(".pipelineInits", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeEvents <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineEvents))
+  assignInMyNamespace(".pipelineEvents", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeParams <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineParams))
+  assignInMyNamespace(".pipelineParams", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeKeep <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineKeep))
+  assignInMyNamespace(".pipelineKeep", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeThetaMat <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineThetaMat))
+  assignInMyNamespace(".pipelineThetaMat", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeOmega <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineOmega))
+  assignInMyNamespace(".pipelineOmega", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeSigma <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineSigma))
+  assignInMyNamespace(".pipelineSigma", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeDfObs <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineDfObs))
+  assignInMyNamespace(".pipelineDfObs", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeDfSub <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineDfSub))
+  assignInMyNamespace(".pipelineDfSub", obj)
+  return(invisible(obj))
+}
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeNSub <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineNSub))
+  assignInMyNamespace(".pipelineNSub", obj)
+  return(invisible(obj))
+}
+
+
+#' @rdname dot-pipeRx
+#' @export
+.pipeNStud <- function(obj) {
+  if (.isNa1(obj)) return(invisible(.pipelineNStud))
+  assignInMyNamespace(".pipelineNStud", obj)
+  return(invisible(obj))
+}
+
 #' Clear/Set pipeline
 #'
-#' @inheritParams rxControl
 #' @param rx rxode2 object
 #' @keywords internal
 #' @return None, clears rxode2 pipeline
@@ -189,6 +317,15 @@ et.rxode2 <- function(x, ..., envir = parent.frame()) {
   assignInMyNamespace(".pipelineRx", x)
   do.call(et, c(list(...), list(envir = envir)), envir = envir)
 }
+
+#' @rdname et
+#' @export
+et.function <- et.rxode2
+
+#' @rdname et
+#' @export
+et.rxUi <- et.rxode2
+
 #' @rdname et
 #' @export
 et.rxSolve <- function(x, ..., envir = parent.frame()) {
@@ -386,6 +523,7 @@ et.default <- function(x, ..., time, amt, evid, cmt, ii, addl,
       .df <- data.frame(time = time, amt = amt)
       ##
       if (!missing(id)) {
+        force(id)
         .df$id <- id
       }
       if (missing(cmt)) {
@@ -524,8 +662,16 @@ et.default <- function(x, ..., time, amt, evid, cmt, ii, addl,
   }
   if (!missing(cmt)) {
     .cmt <- as.character(substitute(cmt))
+    .cmt2 <- try(force(cmt), silent=TRUE)
+    if (inherits(.cmt2, "character") ||
+          inherits(.cmt2, "numeric")) {
+      .cmt <- .cmt2
+    }
     if (length(.cmt) != 1) {
-      if (all(.cmt == .cmt[1])) {
+      if (.cmt[1] == "$") {
+        force(cmt)
+        .cmt <- cmt
+      } else if (all(.cmt == .cmt[1])) {
         .cmt <- .cmt[1]
       } else {
         .cmt0 <- suppressWarnings(try(as.numeric(cmt), silent = TRUE))
@@ -601,6 +747,11 @@ et.default <- function(x, ..., time, amt, evid, cmt, ii, addl,
   .lst <- lapply(.lst, function(x) {
     eval(x, envir)
   })
+  if (any(names(.lst) == "evid")) {
+    if (all(.lst$evid == 0)) {
+      .lst <- .lst[names(.lst) != "evid"]
+    }
+  }
   .Call(`_rxode2_et_`, .lst, list())
 }
 
@@ -608,6 +759,23 @@ et.default <- function(x, ..., time, amt, evid, cmt, ii, addl,
 `$.rxEt` <- function(obj, arg, exact = FALSE) {
   return(.Call(`_rxode2_etUpdate`, obj, arg, NULL, exact))
 }
+#' Dispatch solve to 'rxode2' solve
+#'
+#'
+#' @param x rxode2 solve dispatch object
+#' @param ...  other arguments
+#' @return if 'rxode2'  is loaded, a solved object, otherwise an error
+#' @author Matthew L. Fidler
+#' @export
+rxEtDispatchSolve <- function(x, ...) {
+  UseMethod("rxEtDispatchSolve")
+}
+
+#' @rdname rxEtDispatchSolve
+#' @export
+rxEtDispatchSolve.default <- function(x, ...) {
+  stop("need 'rxode2' loaded for piping to a simulation")
+}
 
 #' @export
 simulate.rxEt <- # nolint
@@ -618,7 +786,9 @@ simulate.rxEt <- # nolint
       if (!is.null(seed)) set.seed(seed)
       return(.Call(`_rxode2_et_`, list(simulate = TRUE), object))
     } else {
-      return(rxSolve(object, ..., seed = seed, nsim = nsim))
+      .ret <- list(object, ..., seed = seed, nsim = nsim)
+      class(.ret) <- "rxode2et"
+      return(rxEtDispatchSolve(.ret))
     }
   }
 
@@ -649,7 +819,7 @@ set_units.rxEt <- function(x, value, ..., mode = .setUnitsMode()) {
       .Call(`_rxode2_et_`, list(amountUnits = "", timeUnits = ""), x)
     }))
   } else {
-    if (!rxIs(value, "character")) value <- deparse(value)
+    if (!inherits(value, "character")) value <- deparse(value)
     .tUnit <- units::set_units(1, "sec", mode = "standard")
     .isTime <- try(units::set_units(units::set_units(1, value, mode = "standard"), "sec"), silent = TRUE)
     if (inherits(.isTime, "try-error")) {
@@ -801,7 +971,7 @@ add.sampling <- function(eventTable, time, time.units = NA) {
 #'
 #' @author Matthew Fidler, Melissa Hallow and Wenping Wang
 #'
-#' @seealso [et()], [rxode2()]
+#' @seealso [et()]
 #'
 #' @examples
 #' # create dosing and observation (sampling) events
@@ -1035,9 +1205,10 @@ as.et.default <- function(x, ...) {
   .e$import.EventTable(as.data.frame(x))
   return(.e)
 }
+
 #' @export
 as.data.frame.rxEt <- function(x, row.names = NULL, optional = FALSE, ...) {
-  if (rxIs(x, "rxEt")) {
+  if (.isRxEt(x)) {
     .x <- x
     .tmp <- .x[, .x$show, drop = FALSE]
     class(.tmp) <- c("rxEt2", "data.frame")
@@ -1071,7 +1242,7 @@ as.data.table.rxEt <- function(x, keep.rownames = FALSE, ...) {
 #' @noRd
 as_tibble.rxEt <- function(x, ...) {
   rxReq("tibble")
-  if (rxIs(x, "rxEt")) {
+  if (.isRxEt(x)) {
     .x <- x
     .show <- .x$show
     class(.x) <- "data.frame"
@@ -1092,10 +1263,11 @@ as_tibble.rxEt <- function(x, ...) {
 #' @return Boolean indicating if this is a rxode2 event table
 #'
 #' @author Matthew L.Fidler
+#'
 #' @export
 #' @keywords internal
 is.rxEt <- function(x) {
-  .Call(`_rxode2_rxIs`, x, "rxEt")
+  .Call(`_rxode2_rxIsEt2`, x)
 }
 #' Expand additional doses
 #'
