@@ -2,6 +2,7 @@ _getRxSolve_t _getRxSolve_;
 _simfun simeps;
 _simfun simeta;
 
+_udf_type _evalUdf = NULL;
 rx_solve *_solveData = NULL;
 rxode2_assign_ptr _assign_ptr = NULL;
 _rxRmModelLibType _rxRmModelLib = NULL;
@@ -37,7 +38,6 @@ rxode2_fn2 gammaqInv;
 rxode2_fn2 gammaqInva;
 
 rxode2i_fn2 rxnorm;
-rxode2i_fn2 rxnormV;
 rxode2i_rxbinom rxbinom;
 rxode2i_rxbinom rxnbinom;
 rxode2i_rxbinom rxnbinomMu;
@@ -54,7 +54,6 @@ rxode2i_fn2 rxunif;
 rxode2i_fn2 rxweibull;
 
 rxode2i2_fn2 rinorm;
-rxode2i2_fn2 rinormV;
 rxode2i2_ribinom ribinom;
 rxode2i2_ribinom rinbinom;
 rxode2i2_ribinom rinbinomMu;
@@ -137,6 +136,16 @@ double _prod(double *input, double *p, int type, int n, ...){
   return _prodPS(input, p, n, type);
 }
 
+double _udf(const char *funName, double *input, int n, ...) {
+  va_list valist;
+  va_start(valist, n);
+  for (unsigned int i = 0; i < n; i++){
+    input[i] = va_arg(valist, double);
+  }
+  va_end(valist);
+  return _evalUdf(funName, n, input);
+}
+
 double _sum(double *input, double *pld, int m, int type, int n, ...){
   va_list valist;
   va_start(valist, n);
@@ -229,9 +238,11 @@ double _transit4P(int cmt, double t, unsigned int id, double n, double mtt, doub
   double ktr = (nd+1)/mtt;
   double lktr = _safe_log(nd+1)-_safe_log(mtt);
   double tlast = _solveData->subjects[id].tlastS[cmt];
+  double dose = _solveData->subjects[id].curDoseS[cmt];
+  if (ISNA(dose)) dose = 0.0;
   if (ISNA(tlast)) tlast = 0.0;
   double tad = (t-tlast);
-  return exp(_safe_log(bio*(_solveData->subjects[id].curDoseS[cmt]))+lktr+n*(lktr+_safe_log(tad))-ktr*(tad)-lgamma1p(nd));
+  return exp(_safe_log(bio*dose)+lktr+n*(lktr+_safe_log(tad))-ktr*(tad)-lgamma1p(nd));
 }
 
 double _transit3P(int cmt, double t, unsigned int id, double n, double mtt){
@@ -242,10 +253,12 @@ double _transit3P(int cmt, double t, unsigned int id, double n, double mtt){
   if (ISNA(tlast)) tlast = 0.0;
   double tad = t-tlast;
   double podo = _solveData->subjects[id].curDoseS[cmt];
+  if (ISNA(podo)) podo = 0.0;
   return exp(_safe_log(podo)+lktr+n*(lktr+_safe_log(tad))-ktr*(tad)-lgamma1p(nd));
 }
 
-void _assignFuns0() {
+void _assignFuns0(void) {
+  _evalUdf = (_udf_type) R_GetCCallable("rxode2", "_rxode2_evalUdf");
   _getRxSolve_ = (_getRxSolve_t) R_GetCCallable("rxode2","getRxSolve_");
   _assign_ptr=(rxode2_assign_ptr) R_GetCCallable("rxode2","rxode2_assign_fn_pointers");
   _rxRmModelLib=(_rxRmModelLibType) R_GetCCallable("rxode2","rxRmModelLib");
@@ -258,45 +271,46 @@ void _assignFuns0() {
   _prodType=(rxode2_fn0i)R_GetCCallable("PreciseSums", "PreciseSums_prod_get");
   _sumType=(rxode2_fn0i)R_GetCCallable("PreciseSums", "PreciseSums_sum_get");
   _ptrid=(rxode2_fn0i)R_GetCCallable("rxode2", "rxode2_current_fn_pointer_id");
+  _compareFactorVal=(rxode2_compareFactorVal_fn) R_GetCCallable("rxode2", "compareFactorVal");
+  _update_par_ptr = (_update_par_ptr_p) R_GetCCallable("rxode2","_update_par_ptr");
+  _getParCov = (_getParCov_p) R_GetCCallable("rxode2","_getParCov");
+  // dynamic start
   linCmtA=(linCmtA_p)R_GetCCallable("rxode2", "linCmtA");
   linCmtB=(linCmtB_p)R_GetCCallable("rxode2", "linCmtB");
   linCmtC=(linCmtA_p)R_GetCCallable("rxode2", "linCmtC");
-    
+
   rxnorm = (rxode2i_fn2)R_GetCCallable("rxode2", "rxnorm");
-  rxnormV = (rxode2i_fn2)R_GetCCallable("rxode2", "rxnormV");
   rxbinom = (rxode2i_rxbinom)R_GetCCallable("rxode2","rxbinom");
   rxnbinom = (rxode2i_rxbinom)R_GetCCallable("rxode2","rxnbinom");
   rxnbinomMu = (rxode2i_rxbinom)R_GetCCallable("rxode2","rxnbinomMu");
   rxcauchy = (rxode2i_fn2)R_GetCCallable("rxode2","rxcauchy") ;
   rxchisq = (rxode2i_fn)R_GetCCallable("rxode2","rxchisq") ;
   rxexp = (rxode2i_fn)R_GetCCallable("rxode2","rxexp");
-  rxf = (rxode2i_fn2)R_GetCCallable("rxode2","rxf") ;
-  rxgeom = (rxode2i_ifn)R_GetCCallable("rxode2","rxgeom") ;
-  rxgamma = (rxode2i_fn2)R_GetCCallable("rxode2","rxgamma") ;
-  rxbeta = (rxode2i_fn2)R_GetCCallable("rxode2","rxbeta") ;
-  rxpois = (rxode2i_ifn)R_GetCCallable("rxode2","rxpois") ;
-  rxt_ = (rxode2i_fn)R_GetCCallable("rxode2","rxt_") ;
-  rxunif = (rxode2i_fn2)R_GetCCallable("rxode2","rxunif") ;
+  rxf = (rxode2i_fn2)R_GetCCallable("rxode2","rxf");
+  rxgeom = (rxode2i_ifn)R_GetCCallable("rxode2","rxgeom");
+  rxgamma = (rxode2i_fn2)R_GetCCallable("rxode2","rxgamma");
+  rxbeta = (rxode2i_fn2)R_GetCCallable("rxode2","rxbeta");
+  rxpois = (rxode2i_ifn)R_GetCCallable("rxode2","rxpois");
+  rxt_ = (rxode2i_fn)R_GetCCallable("rxode2","rxt_");
+  rxunif = (rxode2i_fn2)R_GetCCallable("rxode2","rxunif");
   rxweibull = (rxode2i_fn2)R_GetCCallable("rxode2","rxweibull");
-
   rinorm = (rxode2i2_fn2)R_GetCCallable("rxode2", "rinorm");
-  rinormV = (rxode2i2_fn2)R_GetCCallable("rxode2", "rinormV");
   ribinom = (rxode2i2_ribinom)R_GetCCallable("rxode2","ribinom");
   rinbinom = (rxode2i2_ribinom)R_GetCCallable("rxode2","rinbinom");
   rinbinomMu = (rxode2i2_ribinom)R_GetCCallable("rxode2","rinbinomMu");
-  ricauchy = (rxode2i2_fn2)R_GetCCallable("rxode2","ricauchy") ;
-  richisq = (rxode2i2_fn)R_GetCCallable("rxode2","richisq") ;
+  ricauchy = (rxode2i2_fn2)R_GetCCallable("rxode2","ricauchy");
+  richisq = (rxode2i2_fn)R_GetCCallable("rxode2","richisq");
   riexp = (rxode2i2_fn)R_GetCCallable("rxode2","riexp");
-  rif = (rxode2i2_fn2)R_GetCCallable("rxode2","rif") ;
-  rigeom = (rxode2i2_ifn)R_GetCCallable("rxode2","rigeom") ;
-  rigamma = (rxode2i2_fn2)R_GetCCallable("rxode2","rigamma") ;
-  ribeta = (rxode2i2_fn2)R_GetCCallable("rxode2","ribeta") ;
-  ripois = (rxode2i2_ifn)R_GetCCallable("rxode2","ripois") ;
-  rit_ = (rxode2i2_fn)R_GetCCallable("rxode2","rit_") ;
-  riunif = (rxode2i2_fn2)R_GetCCallable("rxode2","riunif") ;
+  rif = (rxode2i2_fn2)R_GetCCallable("rxode2","rif");
+  rigeom = (rxode2i2_ifn)R_GetCCallable("rxode2","rigeom");
+  rigamma = (rxode2i2_fn2)R_GetCCallable("rxode2","rigamma");
+  ribeta = (rxode2i2_fn2)R_GetCCallable("rxode2","ribeta");
+  ripois = (rxode2i2_ifn)R_GetCCallable("rxode2","ripois");
+  rit_ = (rxode2i2_fn)R_GetCCallable("rxode2","rit_");
+  riunif = (rxode2i2_fn2)R_GetCCallable("rxode2","riunif");
   riweibull = (rxode2i2_fn2)R_GetCCallable("rxode2","riweibull");
-    
   phi = (rxode2_fn)R_GetCCallable("rxode2","phi");
+
   gammap = (rxode2_fn2) R_GetCCallable("rxode2","gammap");
   gammaq = (rxode2_fn2) R_GetCCallable("rxode2","gammaq");
   gammapInv = (rxode2_fn2) R_GetCCallable("rxode2","gammapInv");
@@ -310,69 +324,85 @@ void _assignFuns0() {
   expit = (rxode2_fn3) R_GetCCallable("rxode2", "expit");
   simeta =(_simfun) R_GetCCallable("rxode2", "simeta");
   simeps =(_simfun) R_GetCCallable("rxode2", "simeps");
-  _compareFactorVal=(rxode2_compareFactorVal_fn) R_GetCCallable("rxode2", "compareFactorVal");
-  _update_par_ptr = (_update_par_ptr_p) R_GetCCallable("rxode2","_update_par_ptr");
-  _getParCov = (_getParCov_p) R_GetCCallable("rxode2","_getParCov");
-  
-  _llikNorm=(rxode2_llikNormFun) R_GetCCallable("rxode2","rxLlikNorm");
-  _llikNormDmean=(rxode2_llikNormFun) R_GetCCallable("rxode2","rxLlikNormDmean");
-  _llikNormDsd=(rxode2_llikNormFun) R_GetCCallable("rxode2","rxLlikNormDsd");
-  
-  _llikPois        = (rxode2_llikPoisFun) R_GetCCallable("rxode2","rxLlikPois");
-  _llikPoisDlambda = (rxode2_llikPoisFun) R_GetCCallable("rxode2","rxLlikPoisDlambda");
 
-  _llikBinom = (rxode2_llikBinomFun) R_GetCCallable("rxode2", "rxLlikBinom");
-  _llikBinomDprob = (rxode2_llikBinomFun) R_GetCCallable("rxode2", "rxLlikBinomDprob");
+  _llikNorm=(rxode2_llikNormFun) R_GetCCallable("rxode2ll","rxLlikNorm");
+  _llikNormDmean=(rxode2_llikNormFun) R_GetCCallable("rxode2ll","rxLlikNormDmean");
+  _llikNormDsd=(rxode2_llikNormFun) R_GetCCallable("rxode2ll","rxLlikNormDsd");
 
-  _llikNbinom = (rxode2_llikBinomFun) R_GetCCallable("rxode2", "rxLlikNbinom");
-  _llikNbinomDprob = (rxode2_llikBinomFun) R_GetCCallable("rxode2", "rxLlikNbinomDprob");
-  
-  _llikNbinomMu = (rxode2_llikBinomFun) R_GetCCallable("rxode2", "rxLlikNbinomMu");
-  _llikNbinomMuDmu = (rxode2_llikBinomFun) R_GetCCallable("rxode2", "rxLlikNbinomMuDmu");
+  _llikPois        = (rxode2_llikPoisFun) R_GetCCallable("rxode2ll","rxLlikPois");
+  _llikPoisDlambda = (rxode2_llikPoisFun) R_GetCCallable("rxode2ll","rxLlikPoisDlambda");
 
-  _llikBeta = (rxode2_llikBetaFun)   R_GetCCallable("rxode2", "rxLlikBeta");
-  _llikBetaDshape1 = (rxode2_llikBetaFun) R_GetCCallable("rxode2", "rxLlikBetaDshape1");
-  _llikBetaDshape2 = (rxode2_llikBetaFun) R_GetCCallable("rxode2", "rxLlikBetaDshape2");
+  _llikBinom = (rxode2_llikBinomFun) R_GetCCallable("rxode2ll", "rxLlikBinom");
+  _llikBinomDprob = (rxode2_llikBinomFun) R_GetCCallable("rxode2ll", "rxLlikBinomDprob");
 
-  _llikT = (rxode2_llikTFun)   R_GetCCallable("rxode2", "rxLlikT");
-  _llikTDdf = (rxode2_llikTFun) R_GetCCallable("rxode2", "rxLlikTDdf");
-  _llikTDmean = (rxode2_llikTFun) R_GetCCallable("rxode2", "rxLlikTDmean");
-  _llikTDsd = (rxode2_llikTFun) R_GetCCallable("rxode2", "rxLlikTDsd");
-  _llikChisq    = (rxode2_llikChisqFun) R_GetCCallable("rxode2", "rxLlikChisq");
-  _llikChisqDdf = (rxode2_llikChisqFun) R_GetCCallable("rxode2", "rxLlikChisqDdf");
+  _llikNbinom = (rxode2_llikBinomFun) R_GetCCallable("rxode2ll", "rxLlikNbinom");
+  _llikNbinomDprob = (rxode2_llikBinomFun) R_GetCCallable("rxode2ll", "rxLlikNbinomDprob");
 
-  _llikExp    = (rxode2_llikExpFun) R_GetCCallable("rxode2", "rxLlikExp");
-  _llikExpDrate = (rxode2_llikExpFun) R_GetCCallable("rxode2", "rxLlikExpDrate");
+  _llikNbinomMu = (rxode2_llikBinomFun) R_GetCCallable("rxode2ll", "rxLlikNbinomMu");
+  _llikNbinomMuDmu = (rxode2_llikBinomFun) R_GetCCallable("rxode2ll", "rxLlikNbinomMuDmu");
 
-  _llikF    = (rxode2_llikFFun) R_GetCCallable("rxode2", "rxLlikF");
-  _llikFDdf1 = (rxode2_llikFFun) R_GetCCallable("rxode2", "rxLlikFDdf1");
-  _llikFDdf2 = (rxode2_llikFFun) R_GetCCallable("rxode2", "rxLlikFDdf2");
+  _llikBeta = (rxode2_llikBetaFun)   R_GetCCallable("rxode2ll", "rxLlikBeta");
+  _llikBetaDshape1 = (rxode2_llikBetaFun) R_GetCCallable("rxode2ll", "rxLlikBetaDshape1");
+  _llikBetaDshape2 = (rxode2_llikBetaFun) R_GetCCallable("rxode2ll", "rxLlikBetaDshape2");
 
-  _llikGeom    = (rxode2_llikGeomFun) R_GetCCallable("rxode2", "rxLlikGeom");
-  _llikGeomDp  = (rxode2_llikGeomFun) R_GetCCallable("rxode2", "rxLlikGeomDp");
+  _llikT = (rxode2_llikTFun)   R_GetCCallable("rxode2ll", "rxLlikT");
+  _llikTDdf = (rxode2_llikTFun) R_GetCCallable("rxode2ll", "rxLlikTDdf");
+  _llikTDmean = (rxode2_llikTFun) R_GetCCallable("rxode2ll", "rxLlikTDmean");
+  _llikTDsd = (rxode2_llikTFun) R_GetCCallable("rxode2ll", "rxLlikTDsd");
+  _llikChisq    = (rxode2_llikChisqFun) R_GetCCallable("rxode2ll", "rxLlikChisq");
+  _llikChisqDdf = (rxode2_llikChisqFun) R_GetCCallable("rxode2ll", "rxLlikChisqDdf");
 
-  _llikUnif        = (rxode2_llikUnifFun) R_GetCCallable("rxode2", "rxLlikUnif");
-  _llikUnifDalpha  = (rxode2_llikUnifFun) R_GetCCallable("rxode2", "rxLlikUnifDalpha");
-  _llikUnifDbeta   = (rxode2_llikUnifFun) R_GetCCallable("rxode2", "rxLlikUnifDbeta");
+  _llikExp    = (rxode2_llikExpFun) R_GetCCallable("rxode2ll", "rxLlikExp");
+  _llikExpDrate = (rxode2_llikExpFun) R_GetCCallable("rxode2ll", "rxLlikExpDrate");
 
-  _llikWeibull        = (rxode2_llikWeibullFun) R_GetCCallable("rxode2", "rxLlikWeibull");
-  _llikWeibullDshape  = (rxode2_llikWeibullFun) R_GetCCallable("rxode2", "rxLlikWeibullDshape");
-  _llikWeibullDscale   = (rxode2_llikWeibullFun) R_GetCCallable("rxode2", "rxLlikWeibullDscale");
+  _llikF    = (rxode2_llikFFun) R_GetCCallable("rxode2ll", "rxLlikF");
+  _llikFDdf1 = (rxode2_llikFFun) R_GetCCallable("rxode2ll", "rxLlikFDdf1");
+  _llikFDdf2 = (rxode2_llikFFun) R_GetCCallable("rxode2ll", "rxLlikFDdf2");
 
-  _llikGamma        = (rxode2_llikGammaFun) R_GetCCallable("rxode2", "rxLlikGamma");
-  _llikGammaDshape  = (rxode2_llikGammaFun) R_GetCCallable("rxode2", "rxLlikGammaDshape");
-  _llikGammaDrate   = (rxode2_llikGammaFun) R_GetCCallable("rxode2", "rxLlikGammaDrate");
+  _llikGeom    = (rxode2_llikGeomFun) R_GetCCallable("rxode2ll", "rxLlikGeom");
+  _llikGeomDp  = (rxode2_llikGeomFun) R_GetCCallable("rxode2ll", "rxLlikGeomDp");
 
-  _llikCauchy        = (rxode2_llikCauchyFun) R_GetCCallable("rxode2", "rxLlikCauchy");
-  _llikCauchyDlocation  = (rxode2_llikCauchyFun) R_GetCCallable("rxode2", "rxLlikCauchyDlocation");
-  _llikCauchyDscale   = (rxode2_llikCauchyFun) R_GetCCallable("rxode2", "rxLlikCauchyDscale");
+  _llikUnif        = (rxode2_llikUnifFun) R_GetCCallable("rxode2ll", "rxLlikUnif");
+  _llikUnifDalpha  = (rxode2_llikUnifFun) R_GetCCallable("rxode2ll", "rxLlikUnifDalpha");
+  _llikUnifDbeta   = (rxode2_llikUnifFun) R_GetCCallable("rxode2ll", "rxLlikUnifDbeta");
 
+  _llikWeibull        = (rxode2_llikWeibullFun) R_GetCCallable("rxode2ll", "rxLlikWeibull");
+  _llikWeibullDshape  = (rxode2_llikWeibullFun) R_GetCCallable("rxode2ll", "rxLlikWeibullDshape");
+  _llikWeibullDscale   = (rxode2_llikWeibullFun) R_GetCCallable("rxode2ll", "rxLlikWeibullDscale");
 
+  _llikGamma        = (rxode2_llikGammaFun) R_GetCCallable("rxode2ll", "rxLlikGamma");
+  _llikGammaDshape  = (rxode2_llikGammaFun) R_GetCCallable("rxode2ll", "rxLlikGammaDshape");
+  _llikGammaDrate   = (rxode2_llikGammaFun) R_GetCCallable("rxode2ll", "rxLlikGammaDrate");
+
+  _llikCauchy        = (rxode2_llikCauchyFun) R_GetCCallable("rxode2ll", "rxLlikCauchy");
+  _llikCauchyDlocation  = (rxode2_llikCauchyFun) R_GetCCallable("rxode2ll", "rxLlikCauchyDlocation");
+  _llikCauchyDscale   = (rxode2_llikCauchyFun) R_GetCCallable("rxode2ll", "rxLlikCauchyDscale");
+  // dynamic stop
   _solveData = _getRxSolve_();
 }
 
-void _assignFuns() {
+void _assignFuns(void) {
   if (_assign_ptr == NULL){
     _assignFuns0();
   }
 }
+
+void __assignFuns2(rx_solve rx,
+                   rx_solving_options op,
+                   t_F f,
+                   t_LAG lag,
+                   t_RATE rate,
+                   t_DUR dur,
+                   t_calc_mtime mtime,
+                   t_ME me,
+                   t_IndF indf,
+                   t_getTime gettime,
+                   t_locateTimeIndex timeindex,
+                   t_handle_evidL handleEvid,
+                   t_getDur getdur) {
+  // assign start
+  static rxode2_assignFuns2 rxode2parse_assignFuns2 = NULL;
+  if (rxode2parse_assignFuns2 == NULL) rxode2parse_assignFuns2 = (rxode2_assignFuns2)(R_GetCCallable("rxode2", "_rxode2_assignFuns2"));
+  rxode2parse_assignFuns2(rx, op, f, lag, rate, dur, mtime, me, indf, gettime, timeindex, handleEvid, getdur);
+  // assign stop
+}
