@@ -13,37 +13,46 @@
 #include <errno.h>
 #ifdef ENABLE_NLS
 #include <libintl.h>
-#define _(String) dgettext ("rxode2", String)
+#define _(String) dgettext ("rxode2parse", String)
 /* replace pkg as appropriate */
 #else
 #define _(String) (String)
 #endif
-#include "../inst/include/rxode2.h"
+#include "../inst/include/rxode2parse.h"
+#include "../inst/include/rxode2_control.h"
 #include "tran.h"
-#include "sbuf.h"
+#include "../inst/include/rxode2parseSbuf.h"
 
 // show_ode = 1 dydt
+#define ode_dydt 1
 // show_ode = 2 Jacobian
+#define ode_jac  2
 // show_ode = 3 Ini statement
+#define ode_ini 3
 // show_ode = 0 LHS
+#define ode_lhs 0
+// show_ode = 4 functional bioavailibility
+#define ode_printaux 4
 // show_ode = 5 functional bioavailibility
+#define ode_fbio 5
 // show_ode == 6 functional lag
+#define ode_lag 6
 // show_ode == 7 functional rate
+#define ode_rate 7
 // show_ode == 8 functional duration
+#define ode_dur 8
 // show_ode == 9 functional mtimes
+#define ode_mtime 9
 // show_ode == 10 ME matrix
+#define ode_mexp 10
 // show_ode == 11 Inductive vector
+#define ode_indLinVec 11
 // show_ode == 12 initialize lhs to last value
 // show_ode == 13 #define lags for lhs values
 // show_ode == 14 #define lags for params/covs
 // show_ode == 15 #define sync lhs for simeps
-// show_ode == 16 #define sync lhs for simeps
-
-#define ode_lhs 0
-#define ode_dydt 1
-#define ode_jac  2
-#define ode_ini 3
 #define ode_simeps 15
+// show_ode == 16 #define sync lhs for simeps
 #define ode_simeta 16
 
 // Scenarios
@@ -94,8 +103,8 @@ static inline int isStateLhsI(int i) {
     int doCont=0;
     for (int j = 0; j < tb.de.n; j++) {
       if (tb.di[j] == i) {
-	if (!tb.idu[j]) doCont = 1;
-	break;
+        if (!tb.idu[j]) doCont = 1;
+        break;
       }
     }
     if (doCont) return 1;
@@ -109,9 +118,10 @@ static inline int shouldSkipPrintLhsI(int scenario, int lhs, int i) {
   case print_paramLags:
     return (tb.lag[i] == notLHS || tb.lh[i] == isState);
   case print_lhsLags:
-    return (tb.lag[i] == 0 ||tb.lh[i] != isLHS);
+    return (tb.lag[i] == 0 || (tb.lh[i] != isLHS && tb.lh[i] != isLHSstr));
   case print_lastLhsValue:
-    return !(tb.lh[i] == isLHS || tb.lh[i] == isLhsStateExtra || tb.lh[i] == isLHSparam);
+    return !(tb.lh[i] == isLHS || tb.lh[i] == isLHSstr ||
+             tb.lh[i] == isLhsStateExtra || tb.lh[i] == isLHSparam);
   }
   return (lhs && tb.lh[i]>0 && tb.lh[i] != isLHSparam);
 }
@@ -144,7 +154,7 @@ static inline void printParamLags(char *buf, int *j) {
   sAppendN(&sbOut, "#define last_", 13);
   doDot(&sbOut, buf);
   sAppend(&sbOut, "1(x) _getParCov(_cSub, _solveData, %d, (&_solveData->subjects[_cSub])->n_all_times - 1)\n", *j);
-  
+
   sAppendN(&sbOut, "#undef lead_", 12);
   doDot(&sbOut, buf);
   sAppendN(&sbOut, "1\n", 2);
@@ -161,14 +171,14 @@ static inline void printParamLags(char *buf, int *j) {
 
   sAppendN(&sbOut, "#undef lag_", 11);
   doDot(&sbOut, buf);
-  sAppendN(&sbOut, "1\n", 2);  
+  sAppendN(&sbOut, "1\n", 2);
   sAppendN(&sbOut, "#define lag_", 12);
   doDot(&sbOut, buf);
   sAppend(&sbOut, "1(x) _getParCov(_cSub, _solveData, %d, (&_solveData->subjects[_cSub])->idx - 1)\n", *j);
 
   sAppendN(&sbOut, "#undef lag_", 11);
   doDot(&sbOut, buf);
-  sAppendN(&sbOut, "\n", 1);  
+  sAppendN(&sbOut, "\n", 1);
   sAppendN(&sbOut, "#define lag_", 12);
   doDot(&sbOut, buf);
   sAppend(&sbOut, "(x,y) _getParCov(_cSub, _solveData, %d, (&_solveData->subjects[_cSub])->idx - (y))\n", *j);
@@ -211,7 +221,7 @@ static inline void printDoubleDeclaration(char *buf) {
       !strcmp("rx_hi_", buf) || !strcmp("rx_low_", buf)){
     sAppendN(&sbOut, "__", 2);
   }
-  sAppendN(&sbOut, ";\n", 2);
+  sAppendN(&sbOut, " = NA_REAL;\n", 12);
 }
 
 static inline void printVoidDeclaration(char *buf) {
@@ -250,16 +260,16 @@ static inline void printSimEta(char *buf, int *j) {
 void prnt_vars(int scenario, int lhs, const char *pre_str, const char *post_str, int show_ode);
 
 static inline void printCModelVars(const char *prefix) {
-  sAppend(&sbOut, "extern SEXP %smodel_vars(){\n  int pro=0;\n", prefix);
+  sAppend(&sbOut, "extern SEXP %smodel_vars(void){\n  int pro=0;\n", prefix);
   sAppend(&sbOut, "  SEXP _mv = PROTECT(_rxGetModelLib(\"%smodel_vars\"));pro++;\n", prefix);
   sAppendN(&sbOut, "  if (!_rxIsCurrentC(_mv)){\n", 28);
-  sAppendN(&sbOut, "    SEXP hash    = PROTECT(allocVector(STRSXP, 1));pro++;\n", 58);
-  sAppend(&sbOut, "#define __doBuf__  sprintf(buf, \"", _mv.o+1);
+  sAppendN(&sbOut, "    SEXP hash    = PROTECT(Rf_allocVector(STRSXP, 1));pro++;\n", 61);
+  sAppend(&sbOut, "#define __doBuf__  snprintf(buf, __doBufN__, \"", _mv.o+1);
   int off=0;
   int off2 = 0;
   for (int i = 0; i < _mv.o; i++){
     if (off != 0 && off % 4095 == 0) {
-      sAppend(&sbOut, "\"); \\\n sprintf(buf+%d, \"", off2);
+      sAppend(&sbOut, "\"); \\\n snprintf(buf+%d, __doBufN__-%d, \"", off2, off2);
     }
     off++;
     off2++;
@@ -286,7 +296,7 @@ static inline void printCModelVars(const char *prefix) {
     }
   }
   sAppendN(&sbOut, "\");\n", 4);
-  sAppend(&sbOut,"    char buf[%d];\n    __doBuf__\n#undef __doBuf__\n", off+1);
+  sAppend(&sbOut,"    char buf[%d];\n#define __doBufN__ %d\n    __doBuf__\n#undef __doBuf__\n#undef __doBufN__\n", off+1, off+1);
   sAppendN(&sbOut,"    SET_STRING_ELT(hash, 0, mkChar(buf));\n", 42);
   sAppendN(&sbOut, "    SEXP lst      = PROTECT(_rxQr(hash));pro++;\n", 48);
   sAppendN(&sbOut, "    _assign_ptr(lst);\n", 22);
@@ -301,8 +311,9 @@ static inline void printCModelVars(const char *prefix) {
 }
 
 static inline void printRInit(const char *libname, const char *libname2, const char *prefix) {
-  sAppend(&sbOut,"\n//Create function to call from R's main thread that assigns the required functions. Sometimes they don't get assigned.\nextern void %sassignFuns(){\n  _assignFuns();\n}\n", prefix);
-  sAppend(&sbOut,"\n//Initialize the dll to match rxode2's calls\nvoid R_init0_%s(){\n  // Get C callables on load; Otherwise it isn't thread safe\n", libname2);
+  sAppend(&sbOut,"\n//Create function to call from R's main thread that assigns the required functions. Sometimes they don't get assigned.\nextern void %sassignFuns(void){\n  _assignFuns();\n}\n", prefix);
+  sAppend(&sbOut,"\n//Initialize the dll to match rxode2's calls\nvoid R_init0_%s(void){\n  // Get C callables on load; Otherwise it isn't thread safe\n", libname2);
+  sAppend(&sbOut, "  R_RegisterCCallable(\"%s\",\"%sassignFuns2\", (DL_FUNC) __assignFuns2);\n", libname, prefix);
   sAppend(&sbOut, "  R_RegisterCCallable(\"%s\",\"%sassignFuns\", (DL_FUNC) %sassignFuns);\n", libname, prefix, prefix);
   sAppend(&sbOut, "  R_RegisterCCallable(\"%s\",\"%sinis\",(DL_FUNC) %sinis);\n", libname, prefix, prefix);
   sAppend(&sbOut, "  R_RegisterCCallable(\"%s\",\"%sdydt\",(DL_FUNC) %sdydt);\n", libname, prefix, prefix);
@@ -347,7 +358,7 @@ void writeSb(sbuf *sbb, FILE *fp);
   writeSb(&sbOut, fpIO);
 
 SEXP _rxode2_codegen(SEXP c_file, SEXP prefix, SEXP libname,
-		    SEXP pMd5, SEXP timeId, SEXP mvLast);
+                          SEXP pMd5, SEXP timeId, SEXP mvLast, SEXP goodFuns);
 
 extern int fullPrint;
 #endif // __CODEGEN_H__
