@@ -14,18 +14,18 @@
 #include <errno.h>
 #ifdef ENABLE_NLS
 #include <libintl.h>
-#define _(String) dgettext ("rxode2", String)
+#define _(String) dgettext ("rxode2parse", String)
 /* replace pkg as appropriate */
 #else
 #define _(String) (String)
 #endif
-#include "../inst/include/rxode2.h"
-#include "sbuf.h"
+#include "../inst/include/rxode2parse.h"
+#include "../inst/include/rxode2parseSbuf.h"
 #include "tran.h"
-#include "ode.h"
+#include "../inst/include/rxode2parseVer.h"
 
-static inline SEXP calcSLinCmt() {
-  SEXP sLinCmt = PROTECT(allocVector(INTSXP,12));
+static inline SEXP calcSLinCmt(void) {
+  SEXP sLinCmt = PROTECT(Rf_allocVector(INTSXP,12));
   INTEGER(sLinCmt)[0] = tb.ncmt;
   INTEGER(sLinCmt)[1] = tb.hasKa;
   INTEGER(sLinCmt)[2] = tb.linB;
@@ -38,7 +38,7 @@ static inline SEXP calcSLinCmt() {
   INTEGER(sLinCmt)[10]= tb.thread;
   INTEGER(sLinCmt)[11]= tb.nLlik;
 
-  SEXP sLinCmtN = PROTECT(allocVector(STRSXP, 12));
+  SEXP sLinCmtN = PROTECT(Rf_allocVector(STRSXP, 12));
   SET_STRING_ELT(sLinCmtN, 0, mkChar("ncmt"));
   SET_STRING_ELT(sLinCmtN, 1, mkChar("ka"));
   SET_STRING_ELT(sLinCmtN, 2, mkChar("linB"));
@@ -51,14 +51,14 @@ static inline SEXP calcSLinCmt() {
   SET_STRING_ELT(sLinCmtN, 9, mkChar("simflg"));
   SET_STRING_ELT(sLinCmtN, 10, mkChar("thread"));
   SET_STRING_ELT(sLinCmtN, 11, mkChar("nLlik"));
-  setAttrib(sLinCmt,   R_NamesSymbol, sLinCmtN);
+  Rf_setAttrib(sLinCmt,   R_NamesSymbol, sLinCmtN);
   UNPROTECT(2);
   return(sLinCmt);
 }
 
-static inline SEXP calcVersionInfo() {
-  SEXP version  = PROTECT(allocVector(STRSXP, 3));
-  SEXP versionn = PROTECT(allocVector(STRSXP, 3));
+static inline SEXP calcVersionInfo(void) {
+  SEXP version  = PROTECT(Rf_allocVector(STRSXP, 3));
+  SEXP versionn = PROTECT(Rf_allocVector(STRSXP, 3));
 
   SET_STRING_ELT(versionn,0,mkChar("version"));
   SET_STRING_ELT(versionn,1,mkChar("repo"));
@@ -67,19 +67,31 @@ static inline SEXP calcVersionInfo() {
   SET_STRING_ELT(version,0,mkChar(__VER_ver__));
   SET_STRING_ELT(version,1,mkChar(__VER_repo__));
   SET_STRING_ELT(version,2,mkChar(__VER_md5__));
-  setAttrib(version,   R_NamesSymbol, versionn);
+  Rf_setAttrib(version,   R_NamesSymbol, versionn);
   UNPROTECT(2);
   return version;
 }
 
-static inline void calcNparamsNlhsNslhs() {
+static inline void calcNparamsNlhsNslhs(void) {
   int sli=0, li=0, pi=0;
   for (int i=0; i<NV; i++) {
     int islhs = tb.lh[i];
-    if (islhs>1 && islhs != isLhsStateExtra && islhs != isLHSparam && islhs != isSuppressedLHS) continue;      /* is a state var */
-    if (islhs == isSuppressedLHS){
+    if (islhs>1 &&
+        islhs != isLhsStateExtra &&
+        islhs != isLHSparam &&
+        islhs != isSuppressedLHS &&
+        islhs != isLHSstr &&
+        islhs != isSuppressedLHSstr) {
+      continue;      /* is a state var */
+    }
+    if (islhs == isSuppressedLHS ||
+        islhs == isSuppressedLHSstr){
       sli++;
-    } else if (islhs == isLHS || islhs == isLhsStateExtra || islhs == isLHSparam){
+    } else if (islhs == isLHS ||
+               islhs == isLHSstr ||
+               islhs == isLhsStateExtra ||
+               islhs == isLHSparam ||
+               islhs == isLHSstr){
       li++;
       if (islhs == isLHSparam) pi++;
     } else {
@@ -91,9 +103,9 @@ static inline void calcNparamsNlhsNslhs() {
   tb.sli=sli;
 }
 
-static inline void calcNextra() {
+static inline void calcNextra(void) {
   int offCmt=0,nExtra = 0;
-  char *buf;
+  char *buf, buf2[200];
   for (int i = 0; i < tb.statei; i++){
     if (offCmt == 0 && tb.idu[i] == 0){
       offCmt = 1;
@@ -102,16 +114,16 @@ static inline void calcNextra() {
     } else if (offCmt == 1 && tb.idu[i] == 1){
       // There is an compartment that doesn't have a derivative
       if (tb.linCmt == 0){
-	char *v = rc_dup_str(buf, 0);
-	sprintf(buf, "compartment '%s' needs differential equations defined", v);
-	updateSyntaxCol();
-	trans_syntax_error_report_fn0(buf);
+        char *v = rc_dup_str(buf, 0);
+        snprintf(buf2, 200, "compartment '%s' needs differential equations defined", v);
+        updateSyntaxCol();
+        trans_syntax_error_report_fn0(buf2);
       } else if (!strcmp("depot", buf) || !strcmp("central", buf)) {
       } else {
-	char *v = rc_dup_str(buf, 0);
-	sprintf(buf, _("compartment '%s' needs differential equations defined"), v);
-	updateSyntaxCol();
-	trans_syntax_error_report_fn0(buf);
+        char *v = rc_dup_str(buf, 0);
+        snprintf(buf2, 200, _("compartment '%s' needs differential equations defined"), v);
+        updateSyntaxCol();
+        trans_syntax_error_report_fn0(buf2);
       }
     } else if (offCmt == 1 && tb.idu[i] == 0){
       nExtra++;
@@ -120,7 +132,7 @@ static inline void calcNextra() {
   tb.nExtra=nExtra;
 }
 
-static inline void calcExtracmt() {
+static inline void calcExtracmt(void) {
   extraCmt = 0;
   if (tb.linCmt){
     if (tb.hasKa){
@@ -128,27 +140,19 @@ static inline void calcExtracmt() {
     } else {
       extraCmt=1;
     }
-    if (tb.hasDepotCmt){
+    if (tb.hasDepotCmt == -1){
       trans_syntax_error_report_fn0(_("'cmt(depot)' does not work with 'linCmt()'"));
     }
-    if (tb.hasCentralCmt) {
+    if (tb.hasCentralCmt == -1) {
       trans_syntax_error_report_fn0("'cmt(central)' does not work with 'linCmt()'");
     }
-  } else {
-    if (tb.hasDepot && rx_syntax_require_ode_first){
-      sPrint(&_bufw2, ODEFIRST, "depot");
-      trans_syntax_error_report_fn0(_bufw2.s);
-    } else if (tb.hasCentral && rx_syntax_require_ode_first){
-      sPrint(&_bufw2, ODEFIRST, "depot");
-      trans_syntax_error_report_fn0(_bufw2.s);
-    }
   }
 }
 
-static inline SEXP calcIniVals() {
+static inline SEXP calcIniVals(void) {
   int pro=0;
-  SEXP inin  = PROTECT(allocVector(STRSXP, tb.isPi + tb.ini_i)); pro++;
-  SEXP ini   = PROTECT(allocVector(REALSXP, tb.isPi + tb.ini_i)); pro++;
+  SEXP inin  = PROTECT(Rf_allocVector(STRSXP, tb.isPi + tb.ini_i)); pro++;
+  SEXP ini   = PROTECT(Rf_allocVector(REALSXP, tb.isPi + tb.ini_i)); pro++;
   char *buf;
   for (int i=tb.isPi + tb.ini_i;i--;) REAL(ini)[i] = NA_REAL;
   int ini_i=0;
@@ -157,9 +161,9 @@ static inline SEXP calcIniVals() {
     buf=tb.ss.line[i];
     if (tb.ini[i] == 1 && tb.lh[i] != isLHS){
       if (tb.isPi && !strcmp("pi", buf)) {
-	redo=1;
-	tb.isPi=0;
-	break;
+        redo=1;
+        tb.isPi=0;
+        break;
       }
       SET_STRING_ELT(inin,ini_i,mkChar(buf));
       REAL(ini)[ini_i++] = tb.iniv[i];
@@ -169,44 +173,116 @@ static inline SEXP calcIniVals() {
     SET_STRING_ELT(inin,ini_i,mkChar("pi"));
     REAL(ini)[ini_i++] = M_PI;
   } else if (redo){
-    inin  = PROTECT(allocVector(STRSXP, tb.ini_i));pro++;
-    ini   = PROTECT(allocVector(REALSXP, tb.ini_i));pro++;
+    inin  = PROTECT(Rf_allocVector(STRSXP, tb.ini_i));pro++;
+    ini   = PROTECT(Rf_allocVector(REALSXP, tb.ini_i));pro++;
     for (int i = tb.ini_i; i--;) REAL(ini)[i] = NA_REAL;
     ini_i=0;
     for (int i = 0; i < NV; i++){
       buf=tb.ss.line[i];
       if (tb.ini[i] == 1 && tb.lh[i] != isLHS){
-	if (tb.isPi && !strcmp("pi", buf)) {
-	  redo=1;
-	  tb.isPi=0;
-	  break;
-	}
-	SET_STRING_ELT(inin,ini_i,mkChar(buf));
-	REAL(ini)[ini_i++] = tb.iniv[i];
+        if (tb.isPi && !strcmp("pi", buf)) {
+          redo=1;
+          tb.isPi=0;
+          break;
+        }
+        SET_STRING_ELT(inin,ini_i,mkChar(buf));
+        REAL(ini)[ini_i++] = tb.iniv[i];
       }
     }
   }
   tb.ini_i = ini_i;
 
-  setAttrib(ini,   R_NamesSymbol, inin);
+  Rf_setAttrib(ini,   R_NamesSymbol, inin);
   UNPROTECT(pro);
   return ini;
 }
 
-static inline void populateStateVectors(SEXP state, SEXP sens, SEXP normState, int *stateRm, SEXP extraState) {
+SEXP orderForderS1(SEXP ordIn);
+
+static inline int sortStateVectorsErrHandle(int prop, int pass, int i) {
+  if (prop == 0 || pass == 1) {
+    return 1;
+  }
+  if ((prop & prop0) != 0) {
+    sAppend(&sbt, "'%s(0)', ", tb.ss.line[tb.di[i]]);
+  }
+  if ((prop & propF) != 0) {
+    sAppend(&sbt, "'f(%s)', ", tb.ss.line[tb.di[i]]);
+  }
+  if ((prop & propAlag) != 0) {
+    sAppend(&sbt, "'alag(%s)', ", tb.ss.line[tb.di[i]]);
+  }
+  if ((prop & propRate) != 0) {
+    sAppend(&sbt, "'rate(%s)', ", tb.ss.line[tb.di[i]]);
+  }
+  if ((prop & propDur) != 0) {
+    sAppend(&sbt, "'dur(%s)', ", tb.ss.line[tb.di[i]]);
+  }
+  // Take off trailing "',
+  sbt.o -= 2;
+  sbt.s[sbt.o] = 0;
+  sAppend(&sbt, " present, but d/dt(%s) not defined\n", tb.ss.line[tb.di[i]]);
+  return 0;
+}
+
+static inline SEXP sortStateVectors(SEXP ordS) {
+  int *ord = INTEGER(ordS);
+  sbt.o = 0; // we can use sbt.o since all the code has already been output
+  sbt.s[0] = 0;
+  for (int i = 0; i < tb.de.n; i++) {
+    int cur = tb.didx[i];
+    int prop = tb.dprop[i];
+    int pass = 0;
+    if (tb.linCmt){
+      if (tb.hasDepotCmt == 1 && !strcmp("depot", tb.ss.line[tb.di[i]])){
+        pass = 1;
+      } else if ((tb.hasCentralCmt == 1 || tb.hasDepotCmt == 1)  &&
+                 !strcmp("central", tb.ss.line[tb.di[i]])) {
+        pass = 1;
+      }
+    }
+    if (cur == 0) {
+      // This has a property without an ODE or cmt() statement; should error here.
+      if (sortStateVectorsErrHandle(prop, pass, i)) continue;
+    } else if (cur < 0) {
+      // This is a compartment only defined by CMT() and is used for
+      // dvid ordering, no properties should be defined.
+      ord[i] = -cur;
+      if (sortStateVectorsErrHandle(prop, pass, i)) continue;
+    } else {
+      ord[i] = cur;
+    }
+  }
+  if (sbt.o != 0) {
+    sbt.o--; // remove last newline
+    sbt.s[sbt.o] = 0;
+    sPrint(&_gbuf, "%s", sbt.s);
+    return R_NilValue;
+  }
+  return orderForderS1(ordS);
+}
+
+static inline void populateStateVectors(SEXP state, SEXP sens, SEXP normState, int *stateRm, SEXP extraState, SEXP stateProp, SEXP sensProp, SEXP normProp, int *ordFp) {
   int k=0, j=0, m=0, p=0;
   char *buf;
+  int *statePropI = INTEGER(stateProp);
+  int *sensPropI = INTEGER(sensProp);
+  int *normPropI = INTEGER(normProp);
   for (int i=0; i<tb.de.n; i++) {                     /* name state vars */
-    buf=tb.ss.line[tb.di[i]];
+    buf=tb.ss.line[tb.di[ordFp[i]-1]] ;
     if (tb.idu[i] == 1){
       if (strncmp(buf,"rx__sens_", 9) == 0){
-	SET_STRING_ELT(sens,j++,mkChar(buf));
-	SET_STRING_ELT(state,k++,mkChar(buf));
-	stateRm[k-1]=tb.idi[i];
+        statePropI[k] = tb.dprop[ordFp[i]-1];
+        sensPropI[j] = tb.dprop[ordFp[i]-1];
+        SET_STRING_ELT(sens,j++,mkChar(buf));
+        SET_STRING_ELT(state,k++,mkChar(buf));
+        stateRm[k-1]=tb.idi[ordFp[i]-1];
       } else {
-	SET_STRING_ELT(normState,m++,mkChar(buf));
-	SET_STRING_ELT(state,k++,mkChar(buf));
-	stateRm[k-1]=tb.idi[i];
+        statePropI[k] = tb.dprop[ordFp[i]-1];
+        normPropI[m] = tb.dprop[ordFp[i]-1];
+        SET_STRING_ELT(normState,m++,mkChar(buf));
+        SET_STRING_ELT(state,k++,mkChar(buf));
+        stateRm[k-1]=tb.idi[ordFp[i]-1];
       }
     } else {
       SET_STRING_ELT(extraState, p++, mkChar(buf));
@@ -224,16 +300,16 @@ static inline void populateDfdy(SEXP dfdy) {
       sPrint(&_bufw,"_THETA_%d_",j);
       if (!strcmp(dy,_bufw.s)){
         sPrint(&_bufw,"THETA[%d]",j);
-	foundIt=1;
-	break;
+        foundIt=1;
+        break;
       }
     }
     if (!foundIt){
       for (int j = 1; j <= tb.maxeta;j++){
-	sPrint(&_bufw,"_ETA_%d_",j);
-	if (!strcmp(dy,_bufw.s)){
-	  sPrint(&_bufw,"ETA[%d]",j);
-	}
+        sPrint(&_bufw,"_ETA_%d_",j);
+        if (!strcmp(dy,_bufw.s)){
+          sPrint(&_bufw,"ETA[%d]",j);
+        }
       }
     }
     if (!foundIt){
@@ -246,15 +322,16 @@ static inline void populateDfdy(SEXP dfdy) {
 }
 
 static inline int assertStateCannotHaveDiff(int islhs, int i, char *buf) {
-  if (islhs>1 && islhs != isLhsStateExtra && islhs != isLHSparam) {
+  if (islhs>1 && islhs != isLhsStateExtra && islhs != isLHSparam &&
+      islhs != isLHSstr) {
     if (tb.lag[i] != 0){
       buf=tb.ss.line[i];
       if (islhs == isState){
-	sPrint(&_bufw, _("state '%s': 'lag', 'lead', 'first', 'last', 'diff' not legal"), buf);
-	trans_syntax_error_report_fn0(_bufw.s);
+        sPrint(&_bufw, _("state '%s': 'lag', 'lead', 'first', 'last', 'diff' not legal"), buf);
+        trans_syntax_error_report_fn0(_bufw.s);
       } else if (islhs == 10 || islhs == 11){
-	sPrint(&_bufw, _("suppress '%s': 'lag', 'lead', 'first', 'last', 'diff' not legal"), buf);
-	trans_syntax_error_report_fn0(_bufw.s);
+        sPrint(&_bufw, _("suppress '%s': 'lag', 'lead', 'first', 'last', 'diff' not legal"), buf);
+        trans_syntax_error_report_fn0(_bufw.s);
       }
     }
     return 1;
@@ -263,13 +340,15 @@ static inline int assertStateCannotHaveDiff(int islhs, int i, char *buf) {
 }
 
 static inline int setLhsAndDualLhsParam(int islhs, SEXP lhs, SEXP params, char *buf,
-				    int *li, int *pi) {
-  if (islhs == isLHS || islhs == isLhsStateExtra || islhs == isLHSparam) {
+                                        int *li, int *pi, SEXP lhsStr) {
+  if (islhs == isLHS || islhs == isLHSstr ||
+      islhs == isLhsStateExtra || islhs == isLHSparam) {
     SET_STRING_ELT(lhs, li[0], mkChar(buf));
+    INTEGER(lhsStr)[li[0]] = islhs == isLHSstr;
     li[0] = li[0]+1;
     if (islhs == isLHSparam) {
       if (!strcmp("CMT", buf)) {
-	tb.hasCmt = 1;
+        tb.hasCmt = 1;
       }
       SET_STRING_ELT(params, pi[0], mkChar(buf));
       pi[0] = pi[0]+1;
@@ -293,9 +372,9 @@ static inline void paramSubThetaEtaToBufw(char *buf) {
     for (int j = 1; j <= tb.maxeta;j++){
       sPrint(&_bufw,"_ETA_%d_",j);
       if (!strcmp(buf, _bufw.s)){
-	sPrint(&_bufw,"ETA[%d]",j);
-	foundIt=1;
-	break;
+        sPrint(&_bufw,"ETA[%d]",j);
+        foundIt=1;
+        break;
       }
     }
   }
@@ -319,12 +398,12 @@ static inline void assertLhsAndDualLhsDiffNotLegal(int islhs, int i, char *buf)
   }
 }
 
-static inline void populateParamsLhsSlhs(SEXP params, SEXP lhs, SEXP slhs) {
+static inline void populateParamsLhsSlhs(SEXP params, SEXP lhs, SEXP slhs, int *interp, SEXP lhsStr) {
   int li=0, pi=0, sli = 0;
   char *buf;
   for (int i=0; i<NV; i++) {
     int islhs = tb.lh[i];
-    if (islhs == isSuppressedLHS){
+    if (islhs == isSuppressedLHS || islhs == isSuppressedLHSstr){
       SET_STRING_ELT(slhs, sli++, mkChar(tb.ss.line[i]));
     }
     buf=tb.ss.line[i];
@@ -332,13 +411,14 @@ static inline void populateParamsLhsSlhs(SEXP params, SEXP lhs, SEXP slhs) {
     if (assertStateCannotHaveDiff(islhs, i, buf)) continue;
     assertLhsAndDualLhsDiffNotLegal(islhs, i, buf);
     /* is a state var */
-    if (!setLhsAndDualLhsParam(islhs, lhs, params, buf, &li, &pi)) {
+    if (!setLhsAndDualLhsParam(islhs, lhs, params, buf, &li, &pi, lhsStr)) {
       paramSubThetaEtaToBufw(buf);
+      interp[pi] = tb.interp[i] + 1; // Makes into a legible factor
       SET_STRING_ELT(params, pi++, mkChar(_bufw.s));
     }
   }
 }
 
-SEXP generateModelVars();
+SEXP generateModelVars(void);
 
 #endif  // __genModelVars_H__
