@@ -27,6 +27,8 @@ extern "C" {
 		ind->cacheME=0;
 		ind->curShift=0.0;
 		ind->lastIsSs2 = false;
+    ind->idxLow=0;
+    ind->idxHi=0;
 		// neq[0] = op->neq
     int ncmt = (op->neq + op->extraCmt);
 		for (int j = ncmt; j--;) {
@@ -38,7 +40,13 @@ extern "C" {
 		}
 		ind->inLhs = inLhs;
 		if (rx->nMtime) calc_mtime(solveid, ind->mtime);
-		for (int j = op->nlhs; j--;) ind->lhs[j] = NA_REAL;
+		for (int j = op->nlhs; j--;) {
+      if (op->lhs_str[j] == 1) {
+        ind->lhs[j] = 1.0; // default is first string defined
+      } else {
+        ind->lhs[j] = NA_REAL;
+      }
+    }
 		if ((inLhs == 0 && op->neq > 0) ||
 				(inLhs == 1 && op->neq == 0 && (rx->nIndSim > 0 || (rx->simflg & 1) != 0 ))) {
 			ind->isIni = 1;
