@@ -7,7 +7,7 @@
 #if defined(__cplusplus)
 extern "C" {
 #endif
-  double get_fkeep(int col, int id, rx_solving_options_ind *ind);
+  double get_fkeep(int col, int id, rx_solving_options_ind *ind, int fid);
   int get_fkeepType(int col);
   SEXP get_fkeepLevels(int col);
   SEXP assign_fkeepAttr(int col, SEXP in);
