@@ -1,18 +1,17 @@
 #define USE_FC_LEN_T
-// [[Rcpp::interfaces(r, cpp)]]
 //#undef NDEBUG
 #define STRICT_R_HEADERS
-#include "rxomp.h"
 #define min2( a , b )  ( (a) < (b) ? (a) : (b) )
 #include <RcppArmadillo.h>
 #include "../inst/include/rxode2.h"
+#include "../inst/include/rxode2parse.h"
 #include <threefry.h>
-#include "seed.h"
 #include <R.h>
 #include "checkmate.h"
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #define _(String) dgettext ("rxode2", String)
+#include "rxomp.h"
 /* replace pkg as appropriate */
 #else
 #define _(String) (String)
@@ -21,8 +20,34 @@ using namespace Rcpp;
 using namespace arma;
 #include "../inst/include/rxode2_as.h"
 #include "threefry.h"
+#include "../inst/include/rxode2random_fillVec.h"
 
-extern rx_solving_options op_global;
+#include <boost/random/normal_distribution.hpp>
+#include <boost/random/negative_binomial_distribution.hpp>
+#include <boost/random/binomial_distribution.hpp>
+#include <boost/random/cauchy_distribution.hpp>
+#include <boost/random/chi_squared_distribution.hpp>
+#include <boost/random/exponential_distribution.hpp>
+#include <boost/random/fisher_f_distribution.hpp>
+#include <boost/random/gamma_distribution.hpp>
+#include <boost/random/geometric_distribution.hpp>
+#include <boost/random/poisson_distribution.hpp>
+#include <boost/random/student_t_distribution.hpp>
+#include <boost/random/weibull_distribution.hpp>
+
+extern "C" {
+  extern rx_solve rx_global;
+  extern rx_solving_options op_global;
+
+}
+
+extern "C" uint32_t getRxSeed1(int ncores);
+extern "C" void setSeedEng1(uint32_t seed);
+extern "C" void setRxSeedFinal(uint32_t seed);
+
+arma::vec getLowerVec(int type, rx_solve* rx);
+arma::vec getUpperVec(int type, rx_solve* rx);
+arma::mat getArmaMat(int type, int csim, rx_solve* rx);
 
 //[[Rcpp::export]]
 SEXP rxRmvn_(NumericMatrix A_, arma::rowvec mu, arma::mat sigma,
@@ -65,7 +90,7 @@ SEXP rxRmvn_(NumericMatrix A_, arma::rowvec mu, arma::mat sigma,
     eng.seed(seed);
 #endif
 
-    std::normal_distribution<> snorm(0.0, 1.0);
+    boost::random::normal_distribution<> snorm(0.0, 1.0);
 
     double acc;
     arma::rowvec work(d);
@@ -146,7 +171,7 @@ void rxRmvn2_(arma::mat& A, arma::rowvec mu, arma::mat sigma,
     eng.seed(seed);
 #endif
 
-  std::normal_distribution<> snorm(0.0, 1.0);
+  boost::random::normal_distribution<> snorm(0.0, 1.0);
 
   double acc;
   arma::rowvec work(d);
@@ -222,7 +247,7 @@ double tn(double l, double u, sitmo::threefry& eng, double tol = 2.05){
   // tol=2.05 # controls switch between methods
   // # threshold can be tuned for maximum speed for each platform
   // case: abs(u-l)>tol, uses accept-reject from randn
-  std::normal_distribution<> rnorm(0.0, 1.0);
+  boost::random::normal_distribution<> rnorm(0.0, 1.0);
   std::uniform_real_distribution<> runif(0.0, 1.0);
   double x=0;
   if (fabs(u - l) > tol){
@@ -676,7 +701,7 @@ arma::mat mvrandn(arma::vec lin, arma::vec uin, arma::mat Sig, int n,
   arma::vec D=Lfull.diag();
   arma::uvec perm=outC.perm;
   if (any(D < 1e-10)){
-    Rf_warningcall(R_NilValue, _("truncated multivariate normal may fail as covariance matrix is singular"));
+    Rf_warningcall(R_NilValue, "%s", _("truncated multivariate normal may fail as covariance matrix is singular"));
   }
   // rescale
   arma::mat L=Lfull.each_col()/D;
@@ -713,7 +738,7 @@ arma::mat mvrandn(arma::vec lin, arma::vec uin, arma::mat Sig, int n,
     }
     iter++;
     if (iter == 1e3){
-      Rf_warningcall(R_NilValue, _("acceptance probability smaller than 0.001"));
+      Rf_warningcall(R_NilValue, "%s", _("acceptance probability smaller than 0.001"));
     } else if (iter> 1e4){
       if (accepted == 0) {
 	stop(_("could not sample from truncated normal"));
@@ -799,7 +824,6 @@ arma::mat rxMvrandn_(NumericMatrix A_,
 
 std::vector<sitmo::threefry> _eng;
 
-void seedEngV(uint32_t seed, int ncores);
 extern "C" void seedEng(int ncores) {
   uint32_t seed = getRxSeed1(ncores);
   _eng.clear();
@@ -809,14 +833,19 @@ extern "C" void seedEng(int ncores) {
     _eng.push_back(eng0);
   }
   seed = getRxSeed1(ncores);
-  seedEngV(seed, ncores);
 }
-extern "C" void setSeedEng1V(uint32_t seed);
 extern "C" void setSeedEng1(uint32_t seed) {
   (_eng[rx_get_thread(op_global.cores)]).seed(seed);
-  setSeedEng1V(seed);
 }
 
+//' This seeds the engine based on the number of cores used in random number generation
+//'
+//' @param ncores is the number of cores to use.
+//' @keywords internal
+//' @return Nothing, called for side effects
+//' @export
+//' @examples
+//' rxSeedEng()
 //[[Rcpp::export]]
 RObject rxSeedEng(int ncores = 1){
   seedEng(ncores);
@@ -825,14 +854,14 @@ RObject rxSeedEng(int ncores = 1){
 
 extern "C" int rxnbinomMu(rx_solving_options_ind* ind, int size, double mu) {
   double p = ((double)size)/(((double)size) + mu);
-  std::negative_binomial_distribution<int> d(size, p);
+  boost::random::negative_binomial_distribution<int> d(size, p);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" int rinbinomMu(rx_solving_options_ind* ind, int id, int size, double mu){
   if (ind->isIni == 1) {
     double p = ((double)size)/(((double)size) + mu);
-    std::negative_binomial_distribution<int> d(size, p);
+    boost::random::negative_binomial_distribution<int> d(size, p);
     ind->simIni[id] = (double) d(_eng[rx_get_thread(op_global.cores)]);
   }
   return (int)ind->simIni[id];
@@ -843,7 +872,7 @@ IntegerVector rxnbinomMu_(int size, double mu, int n, int ncores){
   IntegerVector ret(n);
   int n2 = ret.size();
   double p = ((double)size)/(((double)size) + mu);
-  std::binomial_distribution<int> d(size, p);
+  boost::random::binomial_distribution<int> d(size, p);
   int *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -866,13 +895,13 @@ IntegerVector rxnbinomMu_(int size, double mu, int n, int ncores){
 }
 
 extern "C" int rxnbinom(rx_solving_options_ind* ind, int size, double prob) {
-  std::negative_binomial_distribution<int> d(size, prob);
+  boost::random::negative_binomial_distribution<int> d(size, prob);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" int rinbinom(rx_solving_options_ind* ind, int id, int size, double prob){
   if (ind->isIni == 1) {
-    std::negative_binomial_distribution<int> d(size, prob);
+    boost::random::negative_binomial_distribution<int> d(size, prob);
     ind->simIni[id] = (double) d(_eng[rx_get_thread(op_global.cores)]);
   }
   return (int)ind->simIni[id];
@@ -882,7 +911,7 @@ extern "C" int rinbinom(rx_solving_options_ind* ind, int id, int size, double pr
 IntegerVector rxnbinom_(int size, double prob, int n, int ncores){
   IntegerVector ret(n);
   int n2 = ret.size();
-  std::binomial_distribution<int> d(size, prob);
+  boost::random::binomial_distribution<int> d(size, prob);
   int *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -907,13 +936,13 @@ IntegerVector rxnbinom_(int size, double prob, int n, int ncores){
 
 extern "C" int rxbinom(rx_solving_options_ind* ind, int n, double prob){
   if (!ind->inLhs) return 0;
-  std::binomial_distribution<int> d(n, prob);
+  boost::random::binomial_distribution<int> d(n, prob);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" int ribinom(rx_solving_options_ind* ind, int id, int n, double prob){
   if (ind->isIni == 1) {
-    std::binomial_distribution<int> d(n, prob);
+    boost::random::binomial_distribution<int> d(n, prob);
     ind->simIni[id] = (double) d(_eng[rx_get_thread(op_global.cores)]);
   }
   return (int)ind->simIni[id];
@@ -923,7 +952,7 @@ extern "C" int ribinom(rx_solving_options_ind* ind, int id, int n, double prob){
 IntegerVector rxbinom_(int n0, double prob, int n, int ncores){
   IntegerVector ret(n);
   int n2 = ret.size();
-  std::binomial_distribution<int> d(n0, prob);
+  boost::random::binomial_distribution<int> d(n0, prob);
   int *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -947,13 +976,13 @@ IntegerVector rxbinom_(int n0, double prob, int n, int ncores){
 
 extern "C" double rxcauchy(rx_solving_options_ind* ind, double location, double scale){
   if (!ind->inLhs) return 0;
-  std::cauchy_distribution<double> d(location, scale);
+  boost::random::cauchy_distribution<double> d(location, scale);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" double ricauchy(rx_solving_options_ind* ind, int id, double location, double scale){
   if (ind->isIni == 1) {
-    std::cauchy_distribution<double> d(location, scale);
+    boost::random::cauchy_distribution<double> d(location, scale);
     ind->simIni[id] = d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -963,7 +992,7 @@ extern "C" double ricauchy(rx_solving_options_ind* ind, int id, double location,
 NumericVector rxcauchy_(double location, double scale, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::cauchy_distribution<double> d(location, scale);
+  boost::random::cauchy_distribution<double> d(location, scale);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -988,14 +1017,14 @@ NumericVector rxcauchy_(double location, double scale, int n, int ncores){
 extern "C" double rxchisq(rx_solving_options_ind* ind, double df){
   if (!ind->inLhs) return 0;
   // Non central not supported in C++11
-  std::chi_squared_distribution<double> d(df);
+  boost::random::chi_squared_distribution<double> d(df);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" double richisq(rx_solving_options_ind* ind, int id, double df){
   if (ind->isIni == 1) {
     // Non central not supported in C++11
-    std::chi_squared_distribution<double> d(df);
+    boost::random::chi_squared_distribution<double> d(df);
     ind->simIni[id] = d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1005,7 +1034,7 @@ extern "C" double richisq(rx_solving_options_ind* ind, int id, double df){
 NumericVector rxchisq_(double df, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::chi_squared_distribution<double> d(df);
+  boost::random::chi_squared_distribution<double> d(df);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1029,14 +1058,14 @@ NumericVector rxchisq_(double df, int n, int ncores){
 
 extern "C" double rxexp(rx_solving_options_ind* ind, double rate){
   if (!ind->inLhs) return 0;
-  std::exponential_distribution<double> d(rate);
+  boost::random::exponential_distribution<double> d(rate);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 
 extern "C" double riexp(rx_solving_options_ind* ind, int id, double rate){
   if (ind->isIni) {
-    std::exponential_distribution<double> d(rate);
+    boost::random::exponential_distribution<double> d(rate);
     ind->simIni[id] = d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1046,7 +1075,7 @@ extern "C" double riexp(rx_solving_options_ind* ind, int id, double rate){
 NumericVector rxexp_(double rate, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::exponential_distribution<double> d(rate);
+  boost::random::exponential_distribution<double> d(rate);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1070,13 +1099,13 @@ NumericVector rxexp_(double rate, int n, int ncores){
 
 extern "C" double rxf(rx_solving_options_ind* ind, double df1, double df2){
   if (!ind->inLhs) return 0;
-  std::fisher_f_distribution<double> d(df1, df2);
+  boost::random::fisher_f_distribution<double> d(df1, df2);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" double rif(rx_solving_options_ind* ind, int id, double df1, double df2){
   if (ind->isIni) {
-    std::fisher_f_distribution<double> d(df1, df2);
+    boost::random::fisher_f_distribution<double> d(df1, df2);
     ind->simIni[id] =  d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1086,7 +1115,7 @@ extern "C" double rif(rx_solving_options_ind* ind, int id, double df1, double df
 NumericVector rxf_(double df1, double df2, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::fisher_f_distribution<double> d(df1, df2);
+  boost::random::fisher_f_distribution<double> d(df1, df2);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1111,13 +1140,13 @@ NumericVector rxf_(double df1, double df2, int n, int ncores){
 extern "C" double rxgamma(rx_solving_options_ind* ind, double shape, double rate){
   // R uses rate; C++ uses scale
   if (!ind->inLhs) return 0;
-  std::gamma_distribution<double> d(shape, 1.0/rate);
+  boost::random::gamma_distribution<double> d(shape, 1.0/rate);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" double rigamma(rx_solving_options_ind* ind, int id, double shape, double rate) {
   if (ind->isIni) {
-    std::gamma_distribution<double> d(shape, 1.0/rate);
+    boost::random::gamma_distribution<double> d(shape, 1.0/rate);
     ind->simIni[id] =  d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1127,7 +1156,7 @@ extern "C" double rigamma(rx_solving_options_ind* ind, int id, double shape, dou
 NumericVector rxgamma_(double shape, double rate, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::gamma_distribution<double> d(shape, 1.0/rate);
+  boost::random::gamma_distribution<double> d(shape, 1.0/rate);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1171,8 +1200,8 @@ extern "C" double ribeta(rx_solving_options_ind* ind, int id, double shape1, dou
 NumericVector rxbeta_(double shape1, double shape2, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::gamma_distribution<double> d1(shape1, 1.0);
-  std::gamma_distribution<double> d2(shape2, 1.0);
+  boost::random::gamma_distribution<double> d1(shape1, 1.0);
+  boost::random::gamma_distribution<double> d2(shape2, 1.0);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1197,13 +1226,13 @@ NumericVector rxbeta_(double shape1, double shape2, int n, int ncores){
 
 extern "C" int rxgeom(rx_solving_options_ind* ind, double prob){
   if (!ind->inLhs) return 0;
-  std::geometric_distribution<int> d(prob);
+  boost::random::geometric_distribution<int> d(prob);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" int rigeom(rx_solving_options_ind* ind, int id, double prob){
   if (ind->isIni) {
-    std::geometric_distribution<int> d(prob);
+    boost::random::geometric_distribution<int> d(prob);
     ind->simIni[id] = (double)d(_eng[rx_get_thread(op_global.cores)]);
   }
   return (int)ind->simIni[id];
@@ -1213,7 +1242,7 @@ extern "C" int rigeom(rx_solving_options_ind* ind, int id, double prob){
 IntegerVector rxgeom_(double prob, int n, int ncores){
   IntegerVector ret(n);
   int n2 = ret.size();
-  std::geometric_distribution<int> d(prob);
+  boost::random::geometric_distribution<int> d(prob);
   int *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1238,13 +1267,13 @@ IntegerVector rxgeom_(double prob, int n, int ncores){
 // FIXME rnbinom
 extern "C" double rxnorm(rx_solving_options_ind* ind, double mean, double sd){
   if (!ind->inLhs) return 0.0;
-  std::normal_distribution<double> d(mean, sd);
+  boost::random::normal_distribution<double> d(mean, sd);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" double rinorm(rx_solving_options_ind* ind, int id, double mean, double sd) {
   if (ind->isIni) {
-    std::normal_distribution<double> d(mean, sd);
+    boost::random::normal_distribution<double> d(mean, sd);
     ind->simIni[id] = d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1254,7 +1283,7 @@ extern "C" double rinorm(rx_solving_options_ind* ind, int id, double mean, doubl
 NumericVector rxnorm_(double mean, double sd, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::normal_distribution<double> d(mean, sd);
+  boost::random::normal_distribution<double> d(mean, sd);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1278,13 +1307,13 @@ NumericVector rxnorm_(double mean, double sd, int n, int ncores){
 
 extern "C" int rxpois( rx_solving_options_ind* ind, double lambda){
   if (!ind->inLhs) return 0.0;
-  std::poisson_distribution<int> d(lambda);
+  boost::random::poisson_distribution<int> d(lambda);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" int ripois(rx_solving_options_ind* ind, int id, double lambda){
   if (ind->isIni == 1){
-    std::poisson_distribution<int> d(lambda);
+    boost::random::poisson_distribution<int> d(lambda);
     ind->simIni[id] = d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1294,7 +1323,7 @@ extern "C" int ripois(rx_solving_options_ind* ind, int id, double lambda){
 IntegerVector rxpois_(double lambda, int n, int ncores){
   IntegerVector ret(n);
   int n2 = ret.size();
-  std::poisson_distribution<int> d(lambda);
+  boost::random::poisson_distribution<int> d(lambda);
   int *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1318,13 +1347,13 @@ IntegerVector rxpois_(double lambda, int n, int ncores){
 
 extern "C" double rxt_(rx_solving_options_ind* ind, double df){
   if (!ind->inLhs) return 0.0;
-  std::student_t_distribution<double> d(df);
+  boost::random::student_t_distribution<double> d(df);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 extern "C" double rit_(rx_solving_options_ind* ind, int id, double df){
   if (ind->isIni == 1){
-    std::student_t_distribution<double> d(df);
+    boost::random::student_t_distribution<double> d(df);
     ind->simIni[id] =  d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1334,7 +1363,7 @@ extern "C" double rit_(rx_solving_options_ind* ind, int id, double df){
 NumericVector rxt__(double df, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::student_t_distribution<double> d(df);
+  boost::random::student_t_distribution<double> d(df);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1398,14 +1427,14 @@ NumericVector rxunif_(double low, double hi, int n, int ncores){
 
 extern "C" double rxweibull(rx_solving_options_ind* ind, double shape, double scale){
   if (!ind->inLhs) return 0.0;
-  std::weibull_distribution<double> d(shape, scale);
+  boost::random::weibull_distribution<double> d(shape, scale);
   return d(_eng[rx_get_thread(op_global.cores)]);
 }
 
 
 extern "C" double riweibull(rx_solving_options_ind* ind, int id, double shape, double scale){
   if (ind->isIni) {
-    std::weibull_distribution<double> d(shape, scale);
+    boost::random::weibull_distribution<double> d(shape, scale);
     ind->simIni[id] = d(_eng[rx_get_thread(op_global.cores)]);
   }
   return ind->simIni[id];
@@ -1415,7 +1444,7 @@ extern "C" double riweibull(rx_solving_options_ind* ind, int id, double shape, d
 NumericVector rxweibull_(double shape, double scale, int n, int ncores){
   NumericVector ret(n);
   int n2 = ret.size();
-  std::weibull_distribution<double> d(shape, scale);
+  boost::random::weibull_distribution<double> d(shape, scale);
   double *retD = ret.begin();
 
 #ifdef _OPENMP
@@ -1445,19 +1474,6 @@ bool anyFinite(arma::vec v){
   return false;
 }
 
-arma::vec fillVec(arma::vec& in, int len){
-  if ((int)in.size() == len){
-    return in;
-  } else if ((int)(in.size()) == 1){
-    arma::vec out(len);
-    std::fill_n(out.begin(), len, in[0]);
-    return out;
-  } else {
-    arma::vec z;
-    return z;
-  }
-}
-
 //[[Rcpp::export]]
 SEXP rxRmvn0(NumericMatrix& A_, arma::rowvec mu, arma::mat sigma,
 	     arma::vec lower, arma::vec upper, int ncores=1, bool isChol=false,
@@ -1509,10 +1525,6 @@ void rxRmvnA(arma::mat & A_, arma::rowvec & mu, arma::mat &sigma,
   }
 }
 
-arma::vec getLowerVec(int type, rx_solve* rx);
-arma::vec getUpperVec(int type, rx_solve* rx);
-arma::mat getArmaMat(int type, int csim, rx_solve* rx);
-
 void simvar(double *out, int type, int csim, rx_solve* rx) {
   int n = 0;
   if (type == 0) { // eps
@@ -1530,7 +1542,7 @@ void simvar(double *out, int type, int csim, rx_solve* rx) {
   rxRmvnA(A, mu, sigma, lower, upper, 1, false, 0.4, 2.05, 1e-10, 100);
 }
 extern "C" void simeps(int id) {
-  rx_solve* rx = getRxSolve_();
+  rx_solve* rx = &rx_global;
   rx_solving_options_ind *ind = &(rx->subjects[id]);
   if (ind->inLhs == 1) { // only change while calculating the lhs
     // In this case the par_ptr will be updated with the new values, but they are out of order
@@ -1549,7 +1561,7 @@ extern "C" void simeps(int id) {
 
 
 extern "C" void simeta(int id) {
-  rx_solve* rx = getRxSolve_();
+  rx_solve* rx = &rx_global;
   rx_solving_options_ind *ind = &(rx->subjects[id]);
   if (ind->isIni == 1) { // only initialize at beginning
     // In this case the par_ptr will be updated with the new values, but they are out of order
@@ -1733,7 +1745,7 @@ SEXP rxRmvnSEXP(SEXP nS,
 }
 
 static inline NumericVector rpp_h(int n, double& lambda, double& t0i, double &tmax){
-  std::exponential_distribution<double> d(lambda);
+  boost::random::exponential_distribution<double> d(lambda);
   double t0 = t0i;
   NumericVector ret(n);
   for (int i = 0; i < n; ++i){
@@ -1784,7 +1796,7 @@ NumericVector rpp_(SEXP nS, SEXP lambdaS, SEXP gammaS, SEXP probS, SEXP t0S, SEX
 	int cur = 0;
 	NumericVector ret(n);
 	std::uniform_real_distribution<double> runif(0.0, 1.0);
-	std::exponential_distribution<double> rexp(lambda);
+	boost::random::exponential_distribution<double> rexp(lambda);
 	double ttest;
 	while (cur != n){
 	  ttest = t0 + rexp(_eng[rx_get_thread(op_global.cores)]);
