@@ -31,4 +31,10 @@ d/dt(y3) = a3*y1*y2
     rigid_compare <- rigid_compare[setdiff(names(rigid_compare), "timeId")]
     expect_equal(rigid0_compare, rigid_compare)
   })
+
+  test_that("blank modelvars has same size", {
+    blank <- rxModelVars("")
+    full <-  rxModelVars("y = 1")
+    expect_equal(length(rxModelVars(blank)), length(full))
+  })
 })
