@@ -2,6 +2,7 @@ rxTest({
   warfarin <- nlmixr2data::warfarin
 
   test_that("cmt() syntax makes sense", {
+
     mod <- rxode2({
       a <- 6
       b <- 0.6
@@ -26,8 +27,8 @@ rxTest({
         a <- 6
         b <- 0.6
         cmt(matt) # cmt = 1 now
-        d / dt(intestine) <- -a * intestine
-        d / dt(blood) <- a * intestine - b * blood
+        d/dt(intestine) <- -a * intestine
+        d/dt(blood) <- a * intestine - b * blood
       }))),
       regexp = "compartment 'matt' needs differential equations defined"
     )
@@ -553,4 +554,128 @@ rxTest({
     expect_equal(tmp$stateExtra, "cp")
   })
 
+  test_that("compartment ordering does not affect c-code generation #785",
+  {
+
+    f1 <- function() {
+      ini({
+        TVCL <- c(0, 0.0308628127403366)
+        TVV1 <- c(0, 0.132584062223269)
+        TVQ <- c(0, 0.0107718034646091)
+        TVV2 <- c(0, 0.0613517707801907)
+        TVR0 <- c(0, 3.94111989610973)
+        TVKON <- c(0, 1.31095109796496)
+        TVKOFF <- c(0, 0.0758072298659602)
+        TVKDEG <- c(0, 1.07290072658165)
+        TVKINT <- c(0, 3.49422759360926)
+        ADD.ERR.PK <- c(0, 0.446231462752061)
+        ETA.CL ~ 0.0499998244205031
+      })
+      model({
+        CL <- TVCL * exp(ETA.CL)
+        V1 <- TVV1
+        Q <- TVQ
+        V2 <- TVV2
+        R0 <- TVR0
+        KON <- TVKON
+        KOFF <- TVKOFF
+        KDEG <- TVKDEG
+        KINT <- TVKINT
+        KEL <- CL/V1
+        K12 <- Q/V1
+        K21 <- Q/V2
+        KD <- KOFF/KON
+        KSYN <- R0 * KDEG
+        A3(0) <- R0
+        KEL <- 0
+        d/dt(A1) = -(K12 + KEL) * A1 + K21 * A2 - KON * A1 *
+          A3 + KOFF * A4 * V1
+        d/dt(A2) = K12 * A1 - K21 * A2
+        d/dt(A3) = KSYN - KDEG * A3 - KON * (A1/V1) * A3 + KOFF *
+          A4
+        d/dt(A4) = KON * (A1/V1) * A3 - (KINT + KOFF) * A4
+        CFREE = A1/V1
+        CTOT = CFREE + A4
+        CP = log(CTOT)
+        CP ~ add(ADD.ERR.PK)
+        SPEC.OUT <- SPEC
+        DGRP.OUT <- DGRP
+        BW.OUT <- BW
+      })
+    }
+
+    f1 <- f1 %>% zeroRe("omega")
+
+    f2 <- function() {
+      ini({
+        TVCL <- c(0, 0.0308628127403366)
+        TVV1 <- c(0, 0.132584062223269)
+        TVQ <- c(0, 0.0107718034646091)
+        TVV2 <- c(0, 0.0613517707801907)
+        TVR0 <- c(0, 3.94111989610973)
+        TVKON <- c(0, 1.31095109796496)
+        TVKOFF <- c(0, 0.0758072298659602)
+        TVKDEG <- c(0, 1.07290072658165)
+        TVKINT <- c(0, 3.49422759360926)
+        ADD.ERR.PK <- c(0, 0.446231462752061)
+        ETA.CL ~ 0.0499998244205031
+      })
+      model({
+        cmt(A1)
+        cmt(A2)
+        cmt(A3)
+        cmt(A4)
+        CL <- TVCL * exp(ETA.CL)
+        V1 <- TVV1
+        Q <- TVQ
+        V2 <- TVV2
+        R0 <- TVR0
+        KON <- TVKON
+        KOFF <- TVKOFF
+        KDEG <- TVKDEG
+        KINT <- TVKINT
+        KEL <- CL/V1
+        K12 <- Q/V1
+        K21 <- Q/V2
+        KD <- KOFF/KON
+        KSYN <- R0 * KDEG
+        A3(0) <- R0
+        KEL <- 0
+        d/dt(A1) = -(K12 + KEL) * A1 + K21 * A2 - KON * A1 *
+          A3 + KOFF * A4 * V1
+        d/dt(A2) = K12 * A1 - K21 * A2
+        d/dt(A3) = KSYN - KDEG * A3 - KON * (A1/V1) * A3 + KOFF *
+          A4
+        d/dt(A4) = KON * (A1/V1) * A3 - (KINT + KOFF) * A4
+        CFREE = A1/V1
+        CTOT = CFREE + A4
+        CP = log(CTOT)
+        CP ~ add(ADD.ERR.PK)
+        SPEC.OUT <- SPEC
+        DGRP.OUT <- DGRP
+        BW.OUT <- BW
+      })
+    }
+
+    f2 <- f2 %>% zeroRe("omega")
+
+    ev     <- eventTable()
+    mybw   <- 70
+    mymw   <- 150000
+    mydgrp <- 50
+    ev$add.dosing(dose=mydgrp*mybw*1000000/mymw ,nbr.doses= 13, dosing.interval= 7)
+    ev$add.sampling(seq(0,13*7,length.out=7))
+
+    ev2 <- ev %>% as.data.frame() %>%
+      dplyr::mutate(BW=mybw, DGRP=mydgrp, SPEC=4)
+
+    s2 <- rxSolve(f2, ev2, returnType="data.frame")
+
+    s1 <- rxSolve(f1, ev2, returnType="data.frame")
+
+    expect_equal(s1[,c("A1", "A2", "A3", "A4")],
+                 s2[,c("A1", "A2", "A3", "A4")])
+
+  })
+
 })
