@@ -46,6 +46,7 @@ rxTest({
       et(id=1:1000)
 
     skip_on_os("windows")
+
     expect_error(rxSolve(rx1, theta, evw,  nSub=100, nStud=10,
                     thetaMat=thetaMat1,
                     ## Match boundaries of problem
@@ -59,7 +60,3 @@ rxTest({
                  NA)
   })
 })
-
-test_that(".nearPD expected null for matrix that cannot become positive definite", {
-  expect_null(.nearPD(matrix(Inf)))
-})
